% Function to train the moe model using the training data. The training is
% done in a batch mode.
% I/P arguments -
%  x - Input training data as a bit matrix
%  c - class labels
%  T - number of iterations
%  teX - test input
%  teC - test class labels
% O/P arguments -
%  model - the trained model
%  l - training likelihood in each iteration of EM
%  pop - the population size in each iteration of EM
%  teErr - test error at each iteration
%  tel - test likelihood at each iteration
function [model l pop teErr tel] = moe(x,c,T,teX,teC)
[N D] = size(x);
M = 3^D;
q = 0.5*ones(1,M);
px_z = match(x);
choose_init = 0;
if (choose_init == 0)
%    pz = 1/M*ones(1,M);
pz = rand(1,M); pz = pz/sum(pz);
%pz = max(px_z);pz = pz/sum(pz);    
else
    pz = initpz(M,D,choose_init);
end;
%pz = initpz2(M,D);
ind0 = find(c == 0);
ind1 = find(c == 1);
rc = repmat(c,1,M);

for iter = 1:T 
    pop(iter) = length(pz);
    l(iter) = likeli(x,c,q,pz,px_z);
    rq = repmat(q,N,1);
    pc_z = (rq.^rc).*((1-rq).^(1-rc));
    pz_xc = pc_z.*px_z.*repmat(pz,N,1);
    pz_xc = pz_xc./(repmat(sum(pz_xc,2),1,M));
    t1 = sum(pz_xc,1);t2 = sum(pz_xc(ind1,:),1);
    ind = intersect(find(t1 == 0),find(t2 == 0));
    t1(ind) = 1;t2(ind) = 0.5;
    q = t2./t1;
    pz = mean(pz_xc);
    p(:,iter) = pz';
    
    model.M = M;
    model.q = q;
    model.pz = pz;
    
    if (exist('teX','var'))
        [predcl cl_ratio pred1 pred0] = predict(model,teX);
        teErr(iter) = 1 - sum(predcl == teC)/length(teC);
        tel(iter) = sum(teC.*log2(pred1) + (1 - teC).*log2(pred0));
    end;
end;

% Function matches input x to the rules in the population and outputs the
% conditional probability p(x | z)
function matcharr = match(x)
[N D] = size(x);
M = 3^D;
rule = changebase(0:(M-1),3,D);
matcharr = zeros(N,M);
wt = ones(M,1);
for i = 1:M
    wt(i) = 2^(-length(find(rule(i,:) == 2))); 
end;
for i = 1:N
    for j = 1:M
        r = rule(j,:);
        ind2 = find(rule(j,:) == 2);
        r(ind2) = x(i,ind2);
        if (all(r == x(i,:)))
            matcharr(i,j) = wt(j);
        end;
    end;
end;

% Computes the likelihood of data x,c given the model parameters
function val = likeli(x,c,q,pz,px_z)
[N M] = size(px_z);
rq = repmat(q,N,1);rc = repmat(c,1,M);
pc_z = (rq.^rc).*((1-rq).^(1-rc));
pzxc = pc_z.*px_z.*repmat(pz,N,1);
val = sum(log(sum(pzxc,2)));

function pz = initpz(M,D,choose_init)
rule = changebase(0:(M-1),3,D);
for i = 1:M
    n = length(find(rule(i,:) == 2));
    if (choose_init == 1)
        pz(i) = D - n + 1;
    else
        pz(i) = n + 1;
    end;
    %pz(i) = exp(-0.5*(D/2 - n)^2);
    %if (n == 0)
    %    pz(i) = 1;
    %else
    %    pz(i) = 0;
    %end;
end;
pz = pz/sum(pz);

function pz = initpz2(M,D)
rules = changebase(0:(M-1),3,D);
for i = 1:M
    r = rules(i,:);
    pz(i) = sum(r == 2);
end;
pz = pz/sum(pz);
